<?php

// Last Updated : 1/04/2021
// Version : 2.0.2

function r_print($array){

	echo '<pre>';
	print_r($array);
	echo '</pre>';

}

/* PDO functions start here */
function makequery($query, $token_vals = array(), $ret = 0){

	global $user, $globals, $l, $theme;
	
	$errmsg = 'Could not make the Query.<br />'.$query."<br>\n";
	
	// If PDO class is available and pdo_conn is made, ONLY then we will do a PDO calls
	if(class_exists('PDO') && !empty($globals['pdo_conn'])){
		try{
			$pdo = $globals['pdo_conn'];
			$stmt = $pdo->prepare($query);
			
			// If it is an array in $token_vals we will pass to execute()
			if(!empty($token_vals) && is_array($token_vals)){
				
				// Clena the array values as it may give an error if the value is not there.
				$token_vals = clean_pdo_vals($token_vals);
				//r_print($token_vals);
				$stmt_result = $stmt->execute($token_vals);
			
			// We can not pass an empty Array to execute
			}else{
				$stmt_result = $stmt->execute();
			}
			
			if(!$stmt_result){
				
				$errinf = $stmt->errorInfo();
				
				// Try to reconnect if lost connection and execute query again, or 
				// if found any error log/echo error and die
				if(empty($ret)){
					// PDO Error handling
					if(!empty($globals['pdo_conn'])){
						// Reconnect mysql server and try query again if error no is 2006
						if($errinf[1] == 2006){
							
							try{
								$globals['pdo_conn'] = new PDO('mysql:host='.$globals['dbhost'].';dbname='.$globals['db'].';charset=utf8', $globals['dbuser'], $globals['dbpass']);
							}catch(PDOException $e){
								vsql_handle_error($errmsg."Error!: " . $e->getMessage(), 0, 2);
							}
							return makequery($query, $token_vals, $ret);
						
						}elseif($errinf[1] == 2014){
							vsql_handle_error($errmsg."<pre>".print_r($errinf, 1)."</pre>", 0, 1);
							
						// If it is error code 1064 then it means someone is trying to inject multiple queries
						}elseif($errinf[1] == 1064){
							vsql_handle_error($errmsg."<pre>".print_r($errinf, 1)."</pre>", 0, 1);
						}else{
							vsql_handle_error($errmsg."<pre>".print_r($errinf, 1)."</pre>", 0, 2);
						}
						
					}
					
				// Do not die or echo just log the error
				}else{
					vsql_handle_error($errmsg."<pre>".print_r($errinf, 1)."</pre>");
				}
			
			}
			
			// This is important as the other functions like fetch_assoc etc depends on this.
			$result = $stmt;
			
		}catch(PDOException $e){
			vsql_handle_error($errmsg."<pre>".print_r($e->getMessage(), 1)."</pre>", 0, 2);
		}
	// Same OLD SCHOOL - MySQL :)
	}else{
		
		// If token values are there ew will have to do a string replace otherwise mysql will fail to execute the query.
		if(!empty($token_vals)){			
			
			// Sort by string length of values.
			uksort($token_vals, 'v_ksort');
			
			foreach($token_vals as $k => $v){
				if(strpos($v, "'") !== false){
					$v = addslashes($v);
				}
				
				$query = str_replace($k, "'".$v."'", $query);
				
			}
		}

		$result = mysqli_query($globals['conn'], $query);
		
		//Didnt get anyresult - DIE
		if(!$result){
			
			if(empty($ret)){
				
				// Reconnect mysql server and try query again if error no is 2006
				if(mysqli_errno($globals['conn']) == 2006){
					
					$globals['conn'] = @mysqli_connect($globals['dbhost'], $globals['dbuser'], $globals['dbpass'], true) or 
						vsql_handle_error($errmsg."Failed to connect to database server<br>\n".mysqli_errno($globals['conn']).']: '.mysqli_error($globals['conn']), 0, 2);
					
					@mysqli_select_db($globals['db'], $globals['conn']) or 
						vsql_handle_error($errmsg."Unable to select database<br>\n".mysqli_errno($globals['conn']).']: '.mysqli_error($globals['conn']), 0, 2);
					
					return makequery($query, $token_vals, $ret);
					
				}else{
					vsql_handle_error($errmsg."<br>\n<br>\nMySQL Error No : ".mysqli_errno($globals['conn'])."<br>\n<br>\nMySQL Error : ".mysqli_error($globals['conn']), 0, 2);
				}
			}else{
				vsql_handle_error($errmsg."<br>\n<br>\nMySQL Error No : ".mysqli_errno($globals['conn'])."<br>\n<br>\nMySQL Error : ".mysqli_error($globals['conn']));
			}
			
		}
		
	}
	
	return $result;

}

/**
* 
*
* @package	Webuzo
* @author	Abhijeet
* @param		Array	$arr array of values which need to be converted as PDO tokens for use for IN clause
* @param		String	$pre (optional) Prefix for tokens if set. (like if passed prefix_ => :prefix_token)
* @return	Array	Array(token, query) of array of tokens and string of token to use in query.
* @since	2.9.9
**/
function vsql_in_tokens($arr = array(), $pre = ''){
	
	if(!is_array($arr)){
		$arr = array($arr);
	}
	
	$tokens = $query = array();
	
	foreach($arr as $k => $v){
		$pk = ':'.$pre.$k;
		$tokens[$pk] = $arr[$k];
		$query[] = $pk;
	}
	
	// We will send -5 if query is empty so that it will not die in IN clause
	if(empty($query)){
		$query[] = '-111';
	}
	
	return array('tokens' => $tokens, 'query' => implode(',', $query));
}


function vsql_handle_error($msg = '', $echo = 0, $die = 0){
	global $globals;
	$msg = $msg."<br>\n";
	die($msg);
}

function insert_and_id($q, $token_vals = array()){

global $globals;
	
	$res = makequery($q, $token_vals);
	
	// We have to find the last id for PDO as well
	if(class_exists('PDO') && !empty($globals['pdo_conn'])){
		return $globals['pdo_conn']->lastInsertId();
	
	// For MySQL it will as old method
	}else{
		return @mysqli_insert_id($globals['conn']);
	}
}

function update_query($q, $token_vals = array()){

global $globals;

	$res = makequery($q, $token_vals);
	
	// We have to find the last id for PDO as well
	if(class_exists('PDO') && !empty($globals['pdo_conn'])){
		$aff = (int) $res->rowCount();
		
	// For MySQL it will as old method
	}else{
		$aff = (int) @mysqli_affected_rows($globals['conn']);
	}
	
	if($aff < 1){		
		return false;		
	}
	
	return $aff;
}

function clean_pdo_vals($token_vals){
	
	$tmp = array();
	foreach($token_vals as $k => $v){
		if(is_null($v)){
			$token_vals[$k] = '';
		}
	}
	
	return $token_vals;
}
function vsql_affected_rows($res){

global $globals;

	// We have to find the last id for PDO as well
	if(class_exists('PDO') && !empty($globals['pdo_conn'])){
		return (int) $res->rowCount();
		
	// For MySQL it will as old method
	}else{
		return (int) @mysqli_affected_rows($globals['conn']);
	}
}

function vsql_fetch_assoc($res){

global $globals;
	if(class_exists('PDO') && !empty($globals['pdo_conn'])){
		return $res->fetch(PDO::FETCH_ASSOC);
	}else{
		return mysqli_fetch_assoc($res);
	}
}

function vsql_fetch_array($res){

global $globals;
	if(class_exists('PDO') && !empty($globals['pdo_conn'])){
		return $res->fetch(PDO::FETCH_NUM);
	}else{
		return mysqli_fetch_array($res);
	}
	
}

function vsql_num_rows($res){

global $globals;
	if(class_exists('PDO') && !empty($globals['pdo_conn'])){
		return $res->rowCount();
	}else{
		return mysqli_num_rows($res);
	}
}

function vsql_get_client_info(){

global $globals;

	if(class_exists('PDO') && !empty($globals['pdo_conn'])){
		return $globals['pdo_conn']->getAttribute(PDO::ATTR_CLIENT_VERSION);
	}else{
		if(function_exists("mysqli_get_client_info")){
			return mysqli_get_client_info();
		}else{
			return '-';
		}
	}
}

/* PDO function ENDS HERE */

// WHMCS Decrypter
class hash_encryption {
	/**
	 * Hashed value of the user provided encryption key
	 * @var	string
	 **/
	var $hash_key;
	/**
	 * String length of hashed values using the current algorithm
	 * @var	int
	 **/	
	var $hash_lenth;
	/**
	 * Switch base64 enconding on / off
	 * @var	bool	true = use base64, false = binary output / input
	 **/	
	var $base64;
	/**
	 * Secret value added to randomize output and protect the user provided key
	 * @var	string	Change this value to add more randomness to your encryption
	 **/	
	var $salt = 'Change this to any secret value you like. "d41d8cd98f00b204e9800998ecf8427e" might be a good example.';
	
	function __construct($key,$base64 = true){
		return $this->hash_encryption($key,$base64);
	}

	/**
	 * Constructor method
	 *
	 * Used to set key for encryption and decryption.
	 * @param	string	$key	Your secret key used for encryption and decryption
	 * @param	boold	$base64	Enable base64 en- / decoding
	 * @return mixed
	 */
	function hash_encryption($key,$base64 = true) {
		// Toggle base64 usage on / off
		$this->base64 = $base64;
		
		// Instead of using the key directly we compress it using a hash function
		$this->hash_key = $this->_hash($key);
		
		// Remember length of hashvalues for later use
		$this->hash_length = strlen($this->hash_key);
	}
		
	/**
	 * Method used for encryption
	 * @param	string	$string	Message to be encrypted
	 * @return string	Encrypted message
	 */
	function encrypt($string) {
		$iv = $this->_generate_iv();
		
		// Clear output
		$out = '';
		
		// First block of output is ($this->hash_hey XOR IV)
		for($c=0;$c < $this->hash_length;$c++) {
			$out .= chr(ord($iv[$c]) ^ ord($this->hash_key[$c]));
		}

		// Use IV as first key
		$key = $iv;
		$c = 0;

		// Go through input string
		while($c < strlen($string)) {
			// If we have used all characters of the current key we switch to a new one
			if(($c != 0) and ($c % $this->hash_length == 0)) {
				// New key is the hash of current key and last block of plaintext
				$key = $this->_hash($key . substr($string,$c - $this->hash_length,$this->hash_length));
			}
			// Generate output by xor-ing input and key character for character
			$out .= chr(ord($key[$c % $this->hash_length]) ^ ord($string[$c]));
			$c++;
		}
		// Apply base64 encoding if necessary
		if($this->base64) $out = base64_encode($out);
		return $out;
	}
	
	/**
	 * Method used for decryption
	 * @param	string	$string	Message to be decrypted
	 * @return string	Decrypted message
	 */
	function decrypt($string) {
		// Apply base64 decoding if necessary
		if($this->base64) $string = base64_decode($string);
		
		// Extract encrypted IV from input
		$tmp_iv = substr($string,0,$this->hash_length);
		
		// Extract encrypted message from input
		$string = substr($string,$this->hash_length,strlen($string) - $this->hash_length);
		$iv = $out = '';
		
		// Regenerate IV by xor-ing encrypted IV from block 1 and $this->hashed_key
		// Mathematics: (IV XOR KeY) XOR Key = IV
		for($c=0;$c < $this->hash_length;$c++) 
		{
			$iv .= chr(ord($tmp_iv[$c]) ^ ord($this->hash_key[$c]));
		}
		// Use IV as key for decrypting the first block cyphertext
		$key = $iv;
		$c = 0;
		
		// Loop through the whole input string
		while($c < strlen($string)) {
			// If we have used all characters of the current key we switch to a new one
			if(($c != 0) and ($c % $this->hash_length == 0)) {
				// New key is the hash of current key and last block of plaintext
				$key = $this->_hash($key . substr($out,$c - $this->hash_length,$this->hash_length));
			}
			// Generate output by xor-ing input and key character for character
			$out .= chr(ord($key[$c % $this->hash_length]) ^ ord($string[$c]));
			$c++;
		}
		return $out;
	}

	/**
	 * Hashfunction used for encryption
	 *
	 * This class hashes any given string using the best available hash algorithm.
	 * Currently support for md5 and sha1 is provided. In theory even crc32 could be used
	 * but I don't recommend this.
	 *
	 * @access	private
	 * @param	string	$string	Message to hashed
	 * @return string	Hash value of input message
	 */
	function _hash($string) {
		// Use sha1() if possible, php versions >= 4.3.0 and 5
		if(function_exists('sha1')) {
			$hash = sha1($string);
		} else {
			// Fall back to md5(), php versions 3, 4, 5
			$hash = md5($string);
		}
		$out ='';
		// Convert hexadecimal hash value to binary string
		for($c=0;$c<strlen($hash);$c+=2) {
			$out .= $this->_hex2chr($hash[$c] . $hash[$c+1]);
		}
		return $out;
	}
	
	/**
	 * Generate a random string to initialize encryption
	 *
	 * This method will return a random binary string IV ( = initialization vector).
	 * The randomness of this string is one of the crucial points of this algorithm as it
	 * is the basis of encryption. The encrypted IV will be added to the encrypted message
	 * to make decryption possible. The transmitted IV will be encoded using the user provided key.
	 *
	 * @todo	Add more random sources.
	 * @access	private
	 * @see function	hash_encryption
	 * @return string	Binary pseudo random string
	 **/
	function _generate_iv() {
		// Initialize pseudo random generator
		srand ((double)microtime()*1000000);
		
		// Collect random data.
		// Add as many "pseudo" random sources as you can find.
		// Possible sources: Memory usage, diskusage, file and directory content...
		$iv  = $this->salt;
		$iv .= rand(0,getrandmax());
		// Changed to serialize as the second parameter to print_r is not available in php prior to version 4.4
		$iv .= serialize($GLOBALS);
		return $this->_hash($iv);
	}
	
	/**
	 * Convert hexadecimal value to a binary string
	 *
	 * This method converts any given hexadecimal number between 00 and ff to the corresponding ASCII char
	 *
	 * @access	private
	 * @param	string	Hexadecimal number between 00 and ff
	 * @return	string	Character representation of input value
	 **/
	function _hex2chr($num) {
		return chr(hexdec($num));
	}
}

// Generates random strings
function generateRandStr($length){
	
	$randstr = "";
	
	for($i = 0; $i < $length; $i++){
	
		$randnum = mt_rand(0,61);
		
		if($randnum < 10){
		
			$randstr .= chr($randnum+48);
			
		}elseif($randnum < 36){
		
			$randstr .= chr($randnum+55);
			
		}else{
		
			$randstr .= chr($randnum+61);
			
		}
		
	}
	
	return strtolower($randstr);
	
}

// Make an API KEY
function make_apikey($key, $pass){
	return $key.md5($pass.$key);
}

// Webuzo Make API Call
function make_api_call($ip, $pass, $path, $data = array(), $post = array()){
	$key = generateRandStr(8);
	$apikey = make_apikey($key, $pass);
	
	$url = 'https://'.$ip.':4085/'.$path;	
	$url .= (strstr($url, '?') ? '' : '?');	
	$url .= '&api=serialize&apikey='.rawurlencode($apikey);
	
	//echo $url.'<br/>';
	
	// Pass some data if there
	if(!empty($data)){
		$url .= '&apidata='.rawurlencode(base64_encode(serialize($data)));
	}
	
	// Set the curl parameters.
	$ch = curl_init();
	curl_setopt($ch, CURLOPT_URL, $url);
		
	// Connection Time OUT
	curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 3);
	
	// You can timeout in one hour max
	curl_setopt($ch, CURLOPT_TIMEOUT, 3600);
	
	// Turn off the server and peer verification (TrustManager Concept).
	curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, FALSE);
	curl_setopt($ch, CURLOPT_SSL_VERIFYHOST, FALSE);
		
	// UserAgent and Cookies
	curl_setopt($ch, CURLOPT_USERAGENT, 'Softaculous');
	
	if(!empty($post)){
		curl_setopt($ch, CURLOPT_POST, 1);
		curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query($post));
	}
	
	curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
	
	// Get response from the server.
	$resp = curl_exec($ch);
	$curl_err = curl_error($ch);
	curl_close($ch);
	
	// The following line is a method to test - commented for security
	//echo $url.'<br>'.$resp;
	
	if(empty($resp)){
		return false;
	}
	
	$r = unserialize($resp);
	
	if(empty($r)){
		return false;
	}
	
	return $r;
}



// Display a message
function message($str){
	echo $str.(empty($n) ? "" : "\n");
}

// Die with an error message
function derror($str){
	$GLOBALS['error'][] = $str;
}

// Complete with a message
function done($str){
	$GLOBALS['done'] = $str;
}

// OPTIONAL REQ
function optREQ($name, $default = ''){

global $error;

	//Check the GETED NAME was GETed
	if(isset($_REQUEST[$name])){
	
		return trim($_REQUEST[$name]);
		
	}else{
		
		return $default;
	
	}

}

// Theme Header
function theme_header(){
	
	global $error;
	
	echo '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml"><head>
<meta http-equiv="Content-Type" content="text/html; charset=UTF-8">
<meta name="keywords" content="softaculous, software">
<title>cPanel to Webuzo WHMCS Converter</title>
<link rel="stylesheet" type="text/css" href="media/style.css">
<script language="javascript" src="media/js.js" type="text/javascript"></script>
</head>
<body>

<div class="loginmap">
<table width="100%" border="0" cellpadding="0" cellspacing="0">	
<tbody><tr>
	<td colspan="3" style="background: url(media/head.gif);background-repeat: repeat-x;background-size: contain;">
	
		<table width="950" align="center" border="0" cellpadding="0" cellspacing="0">	
			<tbody><tr>
				<td height="43" valign="middle" width="100%" align="left">
				<a href="index.php?"><img src="media/logo.png" height="40" alt=""></a>
				</td>
			</tr>
		</tbody></table>
		
	</td>
</tr>

<tr>
<td>&nbsp;</td>
<td valign="top" width="950" align="left">';

}

function theme_footer(){

	echo '</td>
<td>&nbsp;</td>
</tr>
</tbody></table>
</div>

<br>

</body></html>';
	
}

// Show the Theme Form
function product_theme_form(){

global $product_groups;
global $cpanel_products, $webuzo_products, $virt_serid;
global $sid, $cpanel_servers, $cpanel_sg, $accounts, $multi_cpanel_servers;
	
	//r_print($cpanel_sg);
	//r_print($cpanel_servers);
	
	echo '<form accept-charset="UTF-8" action="" method="post" name="loginform">
<br><br>
<table align="center" border="0" cellpadding="8" cellspacing="0">	
	<tbody><tr>			
		<td valign="middle" width="700">
			<center class="loginhead">Servers Product Conversion</center><br><br>
			<table align="center" border="0" cellpadding="10" cellspacing="0">
				<tr>
				<td class="login_txt" valign="top">
					Choose cPanel Server : <br>
					<font style="font-size:9px">Choose the cPanel Server to convert into a Webuzo Server</font>
				</td>
				<td width="60%" valign="top">
				
					<select name="sid">';
					
					foreach($cpanel_sg as $k => $v){
						
						if(!empty($v['name'])){
							echo '<optgroup label="'.$v['name'].'">';
						}
						
						foreach($v['servers'] as $kk => $vv){
						
							echo '<option value="'.$vv.'" '.($_POST['sid'] == $vv ? 'selected="selected"' : '').'>'.$cpanel_servers[$vv]['name'].'</option>';
						
						}
						
						if(!empty($v['name'])){
							echo '</optgroup>';
						}
						
					}
					
					echo '</select>	
				</td>
				</tr>';
	
	if(!empty($cpanel_products) || !empty($sid)){
		
		echo '<tr>
			<td class="login_txt" valign="top">
				Choose cPanel Product : <br>
				<font style="font-size:9px">Choose the cPanel product you will convert. Once converted, this list will be updated and will not show again.
				</font>
				<br>
				<font style="font-size:9px">If there is an existing Webuzo Product which is equal to the cPanel product you want to convert, you can choose now. <b>OR</b> You can choose the <b>Create</b> Option to let this wizard create a matching product.</font>
			</td>
			<td valign="top">
				<table align="center" border="0" cellpadding="10" cellspacing="0">
				<tr>
					<td>cPanel Product</td>
					<td>Choose Webuzo Product</td>
				</tr>';
				
				foreach($cpanel_products as $k => $v){
					
					echo '
					<tr>
						<td>'.$cpanel_products[$k]['name'].' ('.count($cpanel_products[$k]['accounts']).')</td>
						<td>
							<select name="vpid['.$k.']" id="vpid'.$k.'">
							<option value="0">Do nothing</option>
							<option value="-1" '.(@$_POST['vpid'][$k] == -1 ? 'selected="selected"' : '').'>Create</option>';
							
							foreach($webuzo_products as $wk => $wv){
								
								echo '<option value="'.$wk.'" '.($_POST['vpid'][$k] == $wk ? 'selected="selected"' : '').'>'.$wv['name'].' </option>';
								
							}
							
						echo '</select>
						</td>
					</tr>';
					
				}
				
				echo '</table>
			</td>
		</tr>
		
		<tr>
			<td class="login_txt" valign="top">
				Product Name Suffix : <br>
				<font style="font-size:9px">If a Webuzo product is being created, it will use the name of the cPanel Product. You can add a suffix to this from here.
				</font>
			</td>
			<td valign="top">
				<input name="suffix" value="'.$_REQUEST['suffix'].'" />
			</td>
		</tr>';
		
	}
	
	echo '
	</table>

	<br><br>
	<center>
		<input name="login" value="'.(!empty($cpanel_products) ? 'Start Conversion' : 'Load Data').'" class="submit" type="submit">
	</center>

		</td>
	</tr>
</tbody></table>
</form>
<script language="javascript" type="text/javascript"><!-- // --><![CDATA[

$(document).ready(function(){
	
});

// ]]></script>';

	/*
	r_print($cpanel_servers);
	r_print($cpanel_sg);
	echo 'cPanel Server of Plan :'.$solus_serid;
	r_print($cpanel_products);
	r_print($webuzo_products);
	r_print($webuzo_products);
	echo 'Webuzo Server :'.$virt_serid;*/

}

// The main process function
function process(){
	
global $conn, $db_host, $db_username, $db_password, $db_name, $call_theme, $globals;
global $product_groups;
global $cpanel_products, $webuzo_products, $webuzo_products, $virt_serid;
global $sid, $cpanel_servers, $cpanel_sg, $accounts, $multi_cpanel_servers;

	// Check if the PDO is there 
	if(class_exists('PDO')){
		try{
			$globals['pdo_conn'] = new PDO('mysql:host='.$db_host.';dbname='.$db_name.';charset=utf8', $db_username, $db_password, array(PDO::MYSQL_ATTR_MULTI_STATEMENTS => false));
		}catch(PDOException $e){
			die("Error!: " . $e->getMessage());
		}
		
	// If PDO is not there we will have to use the mysql
	}else{
		
		$globals['conn'] = @mysqli_connect($db_host, $db_username, $db_password, $db_name) or die( "Unable to select database");
	}

	$call_theme = 'product_theme_form';
	
	///////////////////////////////
	// Select all cPanel servers
	///////////////////////////////
	
	$res = makequery("SELECT s.*, r.groupid FROM tblservers s
		LEFT JOIN tblservergroupsrel r ON r.serverid = s.id
		WHERE s.`type` = 'cpanel'
		ORDER BY s.id");
	
	// Did we find anything ?
	if(vsql_num_rows($res) < 1){
		return derror("Could not find any cPanel Servers !");
	}
	
	$cpanel_servers = array();
	
	for($i = 0; $row = vsql_fetch_assoc($res); $i++){
		$cpanel_servers[$row['id']] = $row;
		$row['groupid'] = empty($row['groupid']) ? 0 : $row['groupid'];
		$cpanel_sg[$row['groupid']]['servers'][] = $row['id'];
	}
	
	/////////////////////////
	// Load the Server Group 
	/////////////////////////
		
	$res = makequery("SELECT * FROM tblservergroups
					WHERE `id` IN (".implode(', ', array_keys($cpanel_sg)).")");
					
	if(vsql_num_rows($res) < 1){
		derror('The server group could not be found');
		return;
	}
	
	
	for($i = 0; $row = vsql_fetch_assoc($res); $i++){
		$cpanel_sg[$row['id']] += $row;
	}
	
	// Is the Product ID to convert set ?
	if(!optREQ('sid')){
		return;
	}
	
	$sid = (int) optREQ('sid');	
	
	// Is it valid ?
	if(empty($cpanel_servers[$sid])){
		derror('The Server you submitted is invalid');
		return;
	}
	
	/////////////////////////////////////////////////////////
	// Select cPanel's Hosting Accounts with this server ID
	/////////////////////////////////////////////////////////
	
	$res = makequery("SELECT * FROM tblhosting
				WHERE `server` = '".$sid."'");
				
	if(vsql_num_rows($res) < 1){
		derror('Could not load cPanel Hosting Accounts !');
		return;
	}
	
	for($i=0; $row = vsql_fetch_assoc($res); $i++){
		$accounts[$row['id']] = $row;
		$cpanel_products[$row['packageid']]['accounts'][] = $row['id'];
	}
	
	//r_print($cpanel_products);
	
	
	/////////////////////////////////
	// Select cPanel's Products
	/////////////////////////////////
	
	$res = makequery("SELECT * FROM tblproducts
				WHERE id IN (".implode(', ', array_keys($cpanel_products)).")
				AND `type` IN ('hostingaccount', 'reselleraccount')
				AND servertype = 'cpanel'");
				
	if(vsql_num_rows($res) < 1){
		derror('Could not load cPanel Product\'s !');
		return;
	}
		
	for($i = 0; $row = vsql_fetch_assoc($res); $i++){
		$product_fields = $row;
		$cpanel_products[$row['id']] += $row;
	}
	
	
	/////////////////////////////////
	// Select Webuzo Products
	/////////////////////////////////
	
	$res = makequery("SELECT * FROM tblproducts
				WHERE `type` IN ('hostingaccount', 'reselleraccount')
				AND servertype = 'webuzo'");
				
	if(vsql_num_rows($res) > 0){
		for($i = 0; $row = vsql_fetch_assoc($res); $i++){
			unset($cpanel_products[$row['id']]);
			$webuzo_products[$row['id']] = $row;
		}
	}
	
	//////////////////////////////////
	// Are we to create the Product ?
	//////////////////////////////////
	
	if(empty($_REQUEST['vpid'])){
		return;
	}
	
	foreach($_REQUEST['vpid'] as $c => $w){
		
		if($w == 0 || empty($w)){
			continue;
		}
		
		if(empty($cpanel_products[$c])){
			derror('The cPanel product with ID '.$c.' does not exist');
			return;
		}
		
		if($w != -1 && empty($webuzo_products[$w])){
			derror('The Webuzo product with ID '.$w.' does not exist');
			return;
		}
		
		$vpids[$c] = $w;
		
	}
	//r_print($vpids);return;
	
	// You did not select anything !
	if(empty($vpids)){
		derror('Please select a cPanel Product to convert');
		return;
	}
	
	foreach($vpids as $c => $w){
		
		$vpid = (int) $w;
		
		// Do nothing
		if($vpid == 0){
			continue;
		}
		
		// Search for an existing created product
		if($vpid < 0){
	
			$res = makequery("SELECT * FROM tblproducts
				WHERE `type` IN ('hostingaccount', 'reselleraccount')
				AND servertype = 'webuzo'
				AND configoption24 = 'webuzo-converter-".$c."'");
				
			if(vsql_num_rows($res) > 0){
				for($i = 0; $row = vsql_fetch_assoc($res); $i++){
					$w = $vpids[$c] = $vpid = $row['id'];
				}
			}
			
		}
		
		// We have to create
		if($vpid < 0){
			
			// Is a Group Chosen ?
			$pgid = $cpanel_products[$c]['gid'];
			
			$cpanel_pass = $GLOBALS['hasher']->decrypt($cpanel_products[$c]['password']);
			//echo $cpanel_pass;
		
			// Get the version of WHMCS
			$res_ver = makequery("SELECT `value` FROM tblconfiguration WHERE `setting` = 'Version';");			
			$row = vsql_fetch_assoc($res_ver);
			$version = strstr($row['value'], '-', true);
			$tokens = [];
			
			foreach($product_fields as $pk => $pv){
				
				if(in_array($pk, ['id', 'servertype', 'name', 'servergroup', 'created_at', 'updated_at', 'configoption1', 'configoption24'])){
					continue;
				}
				
				if(preg_match('/configoption/is', $pk)){
					$tokens[$pk] = "`".$pk."` = ''";
				}
				
				$tokens[$pk] = "`".$pk."` = '".addslashes($cpanel_products[$c][$pk])."'";
				
			}
			
			// Now we create the Product
			$w = $vpids[$c] = $vpid = insert_and_id("INSERT INTO tblproducts
				SET `name` = '".$cpanel_products[$c]['name'].(optREQ('suffix') ? optREQ('suffix') : '')."',
				`servertype` = 'webuzo',
				`servergroup` = '".(int)$cpanel_servers[$sid]['groupid']."',
				".implode(', ', $tokens).",
				`configoption1` = '".$cpanel_products[$c]['configoption1']."',
				`configoption24` = 'webuzo-converter-".$c."',
				`created_at` = '".date('Y-m-d H:i:s')."',
				`updated_at` = '".date('Y-m-d H:i:s')."'
			");
				
			$res = makequery("SELECT * FROM tblpricing
				WHERE relid = '".$c."'
				AND `type` = 'product'");
				
			if(vsql_num_rows($res) < 1){
				derror('Could not load cPanel Product\'s pricing for ID - '.$c.' !');
				continue;
			}
			
			$cpanel_pricing = [];
			for($i=0;$i<vsql_num_rows($res);$i++){
				$cpanel_pricing[] = vsql_fetch_assoc($res);
			}
			
			// In case there are two pricing set for product, we have to loop it 
			foreach($cpanel_pricing as $kk => $vv){
				
				// Insert the Pricing
				$res = makequery("INSERT INTO tblpricing
					SET `type` = 'product',
					`currency` = '".$vv['currency']."',
					`relid` = '".$vpid."',
					`msetupfee` = '".$vv['msetupfee']."',
					`qsetupfee` = '".$vv['qsetupfee']."',
					`ssetupfee` = '".$vv['ssetupfee']."',
					`asetupfee` = '".$vv['asetupfee']."',
					`bsetupfee` = '".$vv['bsetupfee']."',
					`tsetupfee` = '".$vv['tsetupfee']."',
					`monthly` = '".$vv['monthly']."',
					`quarterly` = '".$vv['quarterly']."',
					`semiannually` = '".$vv['semiannually']."',
					`annually` = '".$vv['annually']."',
					`biennially` = '".$vv['biennially']."',
					`triennially` = '".$vv['triennially']."'"
				);
				
			}
			
			// Insert the Custom Fields - UUID
			$res = makequery("INSERT INTO tblcustomfields
				SET `type` = 'product',
				`relid` = '".$vpid."',
				`fieldname` = 'uuid',
				`fieldtype` = 'text',
				`description` = 'uuid',
				`fieldoptions` = '',
				`regexpr` = '',
				`adminonly` = 'on',
				`required` = '',
				`showorder` = '',
				`showinvoice` = '',
				`sortorder` = '0',
				`created_at` = '".date('Y-m-d H:i:s')."',
				`updated_at` = '".date('Y-m-d H:i:s')."'"
			);
			
			///////////////////////////////////
			// Select cPanels Config Options
			///////////////////////////////////
			
			$res = makequery("SELECT * FROM tblproductconfiglinks
				WHERE pid = '".$c."'");
			
			$cpanel_productconfiglinks = [];
			if(vsql_num_rows($res) > 0){
				for($i = 0; $row = vsql_fetch_assoc($res); $i++){
					$cpanel_productconfiglinks[$row['gid']] = $row['gid'];
				}
			}
			
			// Are there any Config Options ?
			if(!empty($cpanel_productconfiglinks)){
				
				foreach($cpanel_productconfiglinks as $k => $v){
					
					// Link the cPanel ConfigOptions group to Webuzo
					$res = makequery("INSERT INTO tblproductconfiglinks
						SET `pid` = '$vpid',
						`gid` = '".$v."'"
					);
					
				}
				
			}
			
		}
	
		/////////////////////////////////////
		// Select Webuzo's customfields
		/////////////////////////////////////
		
		$res = makequery("SELECT * FROM tblcustomfields
					WHERE relid = '".$vpid."'
					AND `type` = 'product'");
					
		if(vsql_num_rows($res) < 1){
			derror('Could not load Webuzo Product\'s custom fields !');
			return;
		}
		
		$customfields = [];
		for($i = 0; $row = vsql_fetch_assoc($res); $i++){
			$ttt = explode('|', $row['fieldname']);
			$customfields[$vpid][$ttt[0]] = $row['id'];
		}
		
		//r_print($customfields);
		
		if(empty($customfields[$vpid]['uuid'])){
			derror('Could not find the "uuid" custom field for the Webuzo Product - '.$vpid.' ! Have you created the product with the custom field - uuid ?');
			return;
		}
	
	}

	////////////////////////////////////////////////////
	// Now we will have to load the orders and convert
	// tblhosting
	// tblcustomfieldsvalues
	////////////////////////////////////////////////////

	$count = 0;

	foreach($accounts as $hid => $account){
		
		$webuzo_PROD = $vpids[$account['packageid']];
		
		if(empty($webuzo_PROD)){
			continue;
		}
		
		//echo $hid.' - '.$webuzo_PROD.'<br>';
		
		// INSERT the UUID custom field
		$res = makequery("INSERT INTO tblcustomfieldsvalues
					SET fieldid = '".$customfields[$webuzo_PROD]['uuid']."',
					relid = '".$hid."',
					`value` = '".$account['username']."',
					`created_at` = '".date('Y-m-d H:i:s')."',
					`updated_at` = '".date('Y-m-d H:i:s')."'");
	
		// UPDATE orders
		$res = makequery("UPDATE tblhosting
					SET packageid = '".$webuzo_PROD."'
					WHERE id = '".$hid."'");
		
		// Add to count
		$count += 1;
		
	}
	
	done('The conversion was successful - '.$count.' accounts were converted');

}

///////////////
// Variables
///////////////
$conf_folder = '/tmp';
$call_theme = '';

// To run forever
ignore_user_abort(true);
set_time_limit(0);

// Include WHMCS database configuration file
include(dirname(dirname(__FILE__)).'/configuration.php');
		
$key = md5 (md5 ($cc_encryption_hash)) . md5 ($cc_encryption_hash);
$hasher = new hash_encryption($key);

// Start the process
process();

// Show header
theme_header();

// Any errors ?
if(!empty($error)){
	
	echo '<table align="center" style="background-color: #f7aca8;"><tr><td>
	<ul>';
	
	foreach($error as $k => $v){
		echo '<li>'.$v.'</li>';
	}
	
	echo '</ul>
	</td></tr></table>';
	
}

// Any done ?
if(!empty($done)){
	
	echo '<table align="center"><tr><td class="notice">
	'.$done.'
	</td></tr></table>';
	
}

if(!empty($call_theme)){
	call_user_func($call_theme);
}

// Show footers
theme_footer();

